#include <stdafx.h>
#include <windows.h>
#include <tchar.h>
#include <stdio.h>

#define myheapalloc(x) (HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, x))
#define myheapfree(x)  (HeapFree(GetProcessHeap(), 0, x))

typedef BOOL (WINAPI *SetSecurityDescriptorControlFnPtr)(
   IN PSECURITY_DESCRIPTOR pSecurityDescriptor,
   IN SECURITY_DESCRIPTOR_CONTROL ControlBitsOfInterest,
   IN SECURITY_DESCRIPTOR_CONTROL ControlBitsToSet);

DWORD GetAccessMask(char *source)
{
	char *_source =source;
	DWORD mask = 0;
	char lettre;

	while (lettre=_source[0])
	{
		switch (lettre)
		{
			case 'r':	mask |= GENERIC_READ; break;
			case 'w':	mask |= GENERIC_WRITE; break;
			case 'e':	mask |= GENERIC_EXECUTE; break;
			case 'a':	mask |= GENERIC_ALL; break;
			case 'd':	mask |= DELETE; break;
		}
		_source ++;
	}

/*
	Liste de tout les droits possibles:

  STANDARD:

	* Controle total
	* Modifier
	* Lecture et Executiuon
	* Lecture
	* Ecriture
	
  AVANCE:

  * Parcourir le dossier / Executer le fichier
	* Liste du dossier / Lecture de donnes
	* Attributs de lecture
	* Lisre les attributs etendus
	* Creation de fichier / Ecriture de donnes
	* Creation de dossier / ajout de donnees
	* Attributs d'ecriture
	* Ecriture d'attributs etendus
	* Supression de sous-dossiers et de fichiers
	* Suprrimer
	* Autorisations de lecture
	* Modifier les autorisations
	* Appriation
*/
	return mask;
}

BOOL AddAccessRights(TCHAR *lpszFileName, TCHAR *lpszAccountName, DWORD dwAccessMask) 
{

   // SID variables.
   SID_NAME_USE   snuType;
   TCHAR          domain[255];
   DWORD          cbDomain       = 255;
   char			  userSID[255];
   DWORD          cbUserSID      = 255;

   // File SD variables.
   char			  fileSD[1024];
   DWORD          cbFileSD       = 1024;

   // New SD variables.
   SECURITY_DESCRIPTOR  newSD;

   // ACL variables.
   PACL           pACL           = NULL;
   BOOL           fDaclPresent;
   BOOL           fDaclDefaulted;
   ACL_SIZE_INFORMATION AclInfo;

   // New ACL variables.
   char           newACL[255];
   DWORD          cbNewACL       = 0;

   // Temporary ACE.
   LPVOID         pTempAce       = NULL;
   UINT           CurrentAceIndex = 0;

   UINT           newAceIndex = 0;

   // New APIs available only in Windows 2000 and above for setting 
   // SD control
   SetSecurityDescriptorControlFnPtr _SetSecurityDescriptorControl = NULL;

   try 
   {
		ZeroMemory(userSID,255);
		ZeroMemory(domain,255);
		ZeroMemory(fileSD,1024);
		ZeroMemory(newACL,255);
		
      // STEP 1: Get SID of the account name specified.
      if (!LookupAccountName(NULL, lpszAccountName,userSID, &cbUserSID, domain, &cbDomain, &snuType)) throw(1);

      // STEP 2: Get security descriptor (SD) of the file specified.
      if (!GetFileSecurity(lpszFileName, DACL_SECURITY_INFORMATION, fileSD, 1024, &cbFileSD)) throw(2);

      // STEP 3: Initialize new SD.
      if (!InitializeSecurityDescriptor(&newSD, SECURITY_DESCRIPTOR_REVISION)) throw(3);

      // STEP 4: Get DACL from the old SD.
      if (!GetSecurityDescriptorDacl(fileSD, &fDaclPresent, &pACL,&fDaclDefaulted)) throw(4);

      // STEP 5: Get size information for DACL.
      AclInfo.AceCount = 0; // Assume NULL DACL.
      AclInfo.AclBytesFree = 0;
      AclInfo.AclBytesInUse = sizeof(ACL);

      if (pACL == NULL) fDaclPresent = FALSE;

      // If not NULL DACL, gather size information from DACL.
      if (fDaclPresent) 
	  {             
         if (!GetAclInformation(pACL, &AclInfo, sizeof(ACL_SIZE_INFORMATION), AclSizeInformation)) throw(5);
      }


      // STEP 8: Initialize the new ACL.
      if (!InitializeAcl((PACL) &newACL, 255, ACL_REVISION2)) throw(6);

      // 
      // STEP 9 If DACL is present, copy all the ACEs from the old DACL
      // to the new DACL.
      // 
      // The following code assumes that the old DACL is
      // already in Windows 2000 preferred order.  To conform 
      // to the new Windows 2000 preferred order, first we will 
      // copy all non-inherited ACEs from the old DACL to the 
      // new DACL, irrespective of the ACE type.
      // 
      
      newAceIndex = 0;

      if (fDaclPresent && AclInfo.AceCount) 
	  {

         for (CurrentAceIndex = 0; 
               CurrentAceIndex < AclInfo.AceCount;
               CurrentAceIndex++) 
		{

            // STEP 10: Get an ACE.
            if (!GetAce(pACL, CurrentAceIndex, &pTempAce)) throw(7);

            // STEP 11: Check if it is a non-inherited ACE.
            // If it is an inherited ACE, break from the loop so
            // that the new access allowed non-inherited ACE can
            // be added in the correct position, immediately after
            // all non-inherited ACEs.
			
            if (((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags & INHERITED_ACE) break;

            // STEP 12: Skip adding the ACE, if the SID matches
            // with the account specified, as we are going to 
            // add an access allowed ACE with a different access 
            // mask.

            if (EqualSid(&userSID,&(((ACCESS_ALLOWED_ACE *)pTempAce)->SidStart))) continue;

            // STEP 13: Add the ACE to the new ACL.
            if (!AddAce((PACL) &newACL, ACL_REVISION, MAXDWORD, pTempAce,((PACE_HEADER) pTempAce)->AceSize)) throw(8);
            newAceIndex++;
         }
      }

      // STEP 14: Add the access-allowed ACE to the new DACL.
      // The new ACE added here will be in the correct position,
      // immediately after all existing non-inherited ACEs.
      if (!AddAccessAllowedAce((PACL)&newACL, ACL_REVISION2 , dwAccessMask,&userSID)) throw(9);

      // STEP 15: To conform to the new Windows 2000 preferred order,
      // we will now copy the rest of inherited ACEs from the
      // old DACL to the new DACL.

	  /*
      //if (fDaclPresent && AclInfo.AceCount) 
	  {
         for (; CurrentAceIndex < AclInfo.AceCount;CurrentAceIndex++) 
		 {
            // STEP 16: Get an ACE.
            if (!GetAce(pACL, CurrentAceIndex, &pTempAce)) throw (10);

            // STEP 17: Add the ACE to the new ACL.
            if (!AddAce((PACL) &newACL, ACL_REVISION, MAXDWORD, pTempAce,((PACE_HEADER) pTempAce)->AceSize))  throw(11);
         }
      }
	  */
	  // ADDITIONAL STEP 17b: Detection in the case of Directory use
      
	  //You have now rebuilt a new ACL and want to add it to the newly created DACL.
      if (!AddAccessAllowedAce((PACL) &newACL, ACL_REVISION , dwAccessMask,&userSID))  throw(11);

      //If it's directory, we need to add inheritance staff.
      if ((GetFileAttributes(lpszFileName) & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)
	  {
        // Attempt to grab the next ACE which is what we just added.

        if (!GetAce((PACL) &newACL, newAceIndex , &pTempAce)) throw(12);
		
		((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags = OBJECT_INHERIT_ACE | INHERIT_ONLY_ACE;

        // You have a pointer to the ACE. Place it into a structure, so you can get at its size.           
		if (!AddAccessAllowedAce((PACL) &newACL, ACL_REVISION2 , dwAccessMask,&userSID)) throw(13); 

        //add another ACE for files
        if (!GetAce((PACL) &newACL, newAceIndex+1, &pTempAce)) throw(14);
		((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags = CONTAINER_INHERIT_ACE;
	 }

      // STEP 18: Set the new DACL to the new SD.
      if (!SetSecurityDescriptorDacl(&newSD, TRUE, (PACL) &newACL, FALSE))  throw(15);

      // STEP 19: Copy the old security descriptor control flags 
      // regarding DACL automatic inheritance for Windows 2000 or 
      // later where SetSecurityDescriptorControl() API is available
      // in advapi32.dll.
      _SetSecurityDescriptorControl = (SetSecurityDescriptorControlFnPtr)
            GetProcAddress(GetModuleHandle(TEXT("advapi32.dll")),
            "SetSecurityDescriptorControl");
      if (_SetSecurityDescriptorControl)
	  {
         SECURITY_DESCRIPTOR_CONTROL controlBitsOfInterest = 0;
         SECURITY_DESCRIPTOR_CONTROL controlBitsToSet = 0;
         SECURITY_DESCRIPTOR_CONTROL oldControlBits = 0;
         DWORD dwRevision = 0;

         if (!GetSecurityDescriptorControl(fileSD, &oldControlBits,&dwRevision)) throw (16);

         if (oldControlBits & SE_DACL_AUTO_INHERITED) 
		 {
            controlBitsToSet = controlBitsOfInterest = 
				SE_DACL_AUTO_INHERIT_REQ | SE_DACL_AUTO_INHERITED;
         }
         else if (oldControlBits & SE_DACL_PROTECTED) 
		 {
            controlBitsToSet = controlBitsOfInterest = SE_DACL_PROTECTED;
         }
         
         if (controlBitsOfInterest) 
		 {
            if (!_SetSecurityDescriptorControl(&newSD,controlBitsOfInterest,controlBitsToSet)) throw (17);
         }
      }

      // STEP 20: Set the new SD to the File.
      if (!SetFileSecurity(lpszFileName, DACL_SECURITY_INFORMATION,&newSD)) throw(14); 
	}
	catch (int i)
	{
		printf("AddAccessRights failed in point:%d with code:%d\n",i, GetLastError());
		return FALSE;
	}
	return TRUE;
}


int main(int argc, char *argv[]) 
{
   if (argc < 4) 
   {
	  printf("*********************************************************************\n");
	  printf("* ACE - write by alkaine - sebbloc@ifrance.com - www.mignonsoft.com *\n");
	  printf("*********************************************************************\n");
      printf("usage: \"%s\" <FileName> <AccountName> <right: [r|w|e|d|a]>\n", argv[0]);
      return 1;
   }

   return AddAccessRights(argv[1], argv[2], GetAccessMask (argv[3]));
}

