
BOOL AddAccessRights(TCHAR *lpszFileName, TCHAR *lpszAccountName, DWORD dwAccessMask) 
{

   // SID variables.
   SID_NAME_USE   snuType;
   TCHAR *        szDomain       = NULL;
   DWORD          cbDomain       = 0;
   LPVOID         pUserSID       = NULL;
   DWORD          cbUserSID      = 0;

   // File SD variables.
   PSECURITY_DESCRIPTOR pFileSD  = NULL;
   DWORD          cbFileSD       = 0;

   // New SD variables.
   SECURITY_DESCRIPTOR  newSD;

   // ACL variables.
   PACL           pACL           = NULL;
   BOOL           fDaclPresent;
   BOOL           fDaclDefaulted;
   ACL_SIZE_INFORMATION AclInfo;

   // New ACL variables.
   PACL           pNewACL        = NULL;
   DWORD          cbNewACL       = 0;

   // Temporary ACE.
   LPVOID         pTempAce       = NULL;
   UINT           CurrentAceIndex = 0;

   UINT           newAceIndex = 0;

   // Assume function will fail.
   BOOL           fResult        = FALSE;
   BOOL           fAPISuccess;

   SECURITY_INFORMATION secInfo = DACL_SECURITY_INFORMATION;

   // New APIs available only in Windows 2000 and above for setting 
   // SD control
   SetSecurityDescriptorControlFnPtr _SetSecurityDescriptorControl = NULL;

   __try 
   {

      // 
      // STEP 1: Get SID of the account name specified.
      // 
      fAPISuccess = LookupAccountName(NULL, lpszAccountName,
            pUserSID, &cbUserSID, szDomain, &cbDomain, &snuType);

      // API should have failed with insufficient buffer.
      if (fAPISuccess)
         __leave;
      else if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) 
	  {
         _tprintf(TEXT("LookupAccountName() failed. Error %d [1]\n"), GetLastError());
         __leave;
      }

      pUserSID = myheapalloc(cbUserSID);
      if (!pUserSID) 
	  {
         _tprintf(TEXT("HeapAlloc() failed. Error %d [2]\n"), GetLastError());
         __leave;
      }

      szDomain = (TCHAR *) myheapalloc(cbDomain * sizeof(TCHAR));
      if (!szDomain) 
	  {
         _tprintf(TEXT("HeapAlloc() failed. Error %d [3]\n"), GetLastError());
         __leave;
      }

      fAPISuccess = LookupAccountName(NULL, lpszAccountName,
            pUserSID, &cbUserSID, szDomain, &cbDomain, &snuType);
      if (!fAPISuccess) 
	  {
         _tprintf(TEXT("LookupAccountName() failed. Error %d [4]\n"), 
               GetLastError());
         __leave;
      }

      // 
      // STEP 2: Get security descriptor (SD) of the file specified.
      // 
      fAPISuccess = GetFileSecurity(lpszFileName, 
            secInfo, pFileSD, 0, &cbFileSD);

      // API should have failed with insufficient buffer.
      if (fAPISuccess)
         __leave;
      else if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) 
	  {
         _tprintf(TEXT("GetFileSecurity() failed. Error %d [5]\n"), 
               GetLastError());
         __leave;
      }

      pFileSD = myheapalloc(cbFileSD);
      if (!pFileSD) 
	  {
         _tprintf(TEXT("HeapAlloc() failed. Error %d [6]\n"), GetLastError());
         __leave;
      }

      fAPISuccess = GetFileSecurity(lpszFileName, 
            secInfo, pFileSD, cbFileSD, &cbFileSD);
      if (!fAPISuccess) 
	  {
         _tprintf(TEXT("GetFileSecurity() failed. Error %d [7]\n"), 
               GetLastError());
         __leave;
      }

      // 
      // STEP 3: Initialize new SD.
      // 
      if (!InitializeSecurityDescriptor(&newSD, 
            SECURITY_DESCRIPTOR_REVISION)) 
	  {
         _tprintf(TEXT("InitializeSecurityDescriptor() failed. Error %d [8]\n"), GetLastError());
         __leave;
      }

      // 
      // STEP 4: Get DACL from the old SD.
      // 
      if (!GetSecurityDescriptorDacl(pFileSD, &fDaclPresent, &pACL,
            &fDaclDefaulted)) 
	  {
         _tprintf(TEXT("GetSecurityDescriptorDacl() failed. Error %d [9]\n"),
               GetLastError());
         __leave;
      }

      // 
      // STEP 5: Get size information for DACL.
      // 
      AclInfo.AceCount = 0; // Assume NULL DACL.
      AclInfo.AclBytesFree = 0;
      AclInfo.AclBytesInUse = sizeof(ACL);

      if (pACL == NULL) fDaclPresent = FALSE;

      // If not NULL DACL, gather size information from DACL.
      if (fDaclPresent) 
	  {             
         if (!GetAclInformation(pACL, &AclInfo, sizeof(ACL_SIZE_INFORMATION), AclSizeInformation)) 
		 {
            _tprintf(TEXT("GetAclInformation() failed. Error %d [10]\n"),
                  GetLastError());
            __leave;
         }
      }

      // 
      // STEP 6: Compute size needed for the new ACL.
      // 
	  
      cbNewACL = AclInfo.AclBytesInUse + (sizeof(ACCESS_ALLOWED_ACE) + GetLengthSid(pUserSID))*2 - sizeof(DWORD);
	   //cbNewACL = AclInfo.AclBytesInUse + sizeof(ACCESS_ALLOWED_ACE) + GetLengthSid(pUserSID) - sizeof(DWORD);

      // 
      // STEP 7: Allocate memory for new ACL.
      // 
      pNewACL = (PACL) myheapalloc(cbNewACL);
      if (!pNewACL) 
	  {
         _tprintf(TEXT("HeapAlloc() failed. Error %d [11]\n"), GetLastError());
         __leave;
      }

      // 
      // STEP 8: Initialize the new ACL.
      // 
      if (!InitializeAcl(pNewACL, cbNewACL, ACL_REVISION2)) 
	  {
         _tprintf(TEXT("InitializeAcl() failed. Error %d [12]\n"), GetLastError());
         __leave;
      }

      // 
      // STEP 9 If DACL is present, copy all the ACEs from the old DACL
      // to the new DACL.
      // 
      // The following code assumes that the old DACL is
      // already in Windows 2000 preferred order.  To conform 
      // to the new Windows 2000 preferred order, first we will 
      // copy all non-inherited ACEs from the old DACL to the 
      // new DACL, irrespective of the ACE type.
      // 
      
      newAceIndex = 0;

      if (fDaclPresent && AclInfo.AceCount) 
	  {

         for (CurrentAceIndex = 0; 
               CurrentAceIndex < AclInfo.AceCount;
               CurrentAceIndex++) 
		{

            // 
            // STEP 10: Get an ACE.
            // 
            if (!GetAce(pACL, CurrentAceIndex, &pTempAce)) 
			{
               _tprintf(TEXT("GetAce() failed. Error %d [13]\n"), GetLastError());
               __leave;
            }

            // 
            // STEP 11: Check if it is a non-inherited ACE.
            // If it is an inherited ACE, break from the loop so
            // that the new access allowed non-inherited ACE can
            // be added in the correct position, immediately after
            // all non-inherited ACEs.
            // 
			
            if (((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags
               & INHERITED_ACE)
               break;

            // 
            // STEP 12: Skip adding the ACE, if the SID matches
            // with the account specified, as we are going to 
            // add an access allowed ACE with a different access 
            // mask.
            // 
            if (EqualSid(pUserSID,
               &(((ACCESS_ALLOWED_ACE *)pTempAce)->SidStart)))
               continue;

            // 
            // STEP 13: Add the ACE to the new ACL.
            // 
            if (!AddAce(pNewACL, ACL_REVISION, MAXDWORD, pTempAce,((PACE_HEADER) pTempAce)->AceSize)) 
			{
               _tprintf(TEXT("AddAce() failed. Error %d [14]\n"),GetLastError());
               __leave;
            }

            newAceIndex++;
         }
      }

      // 
      // STEP 14: Add the access-allowed ACE to the new DACL.
      // The new ACE added here will be in the correct position,
      // immediately after all existing non-inherited ACEs.
      // 
      if (!AddAccessAllowedAce(pNewACL, ACL_REVISION2 , dwAccessMask,pUserSID)) 
	  {
         _tprintf(TEXT("AddAccessAllowedAce() failed. Error %d [15]\n"), GetLastError());
         __leave;
      }

      // 
      // STEP 15: To conform to the new Windows 2000 preferred order,
      // we will now copy the rest of inherited ACEs from the
      // old DACL to the new DACL.
      // 
      if (fDaclPresent && AclInfo.AceCount) 
	  {
         for (; CurrentAceIndex < AclInfo.AceCount;CurrentAceIndex++) 
		 {
            // 
            // STEP 16: Get an ACE.
            // 
            if (!GetAce(pACL, CurrentAceIndex, &pTempAce)) 
			{
               _tprintf(TEXT("GetAce() failed. Error %d [16]\n"), GetLastError());
               __leave;
            }

            // 
            // STEP 17: Add the ACE to the new ACL.
            // 
            if (!AddAce(pNewACL, ACL_REVISION, MAXDWORD, pTempAce,((PACE_HEADER) pTempAce)->AceSize)) 
			{
               _tprintf(TEXT("AddAce() failed. Error %d [17]\n"), GetLastError());
               __leave;
            }
         }
      }

	  // 
	  // ADDITIONAL STEP 17b: Detection in the case of Directory use
	  //
      
	  //You have now rebuilt a new ACL and want to add it to the newly created DACL.
      if (!AddAccessAllowedAce(pNewACL, ACL_REVISION , dwAccessMask,pUserSID)) 
	  {
         _tprintf(TEXT("AddAccessAllowedAce() failed. Error %d [18]\n"),GetLastError());
         __leave;
      }

      //If it's directory, we need to add inheritance staff.
      if ( (GetFileAttributes(lpszFileName) & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)
	  {
        // Attempt to grab the next ACE which is what we just added.

        if (!GetAce(pNewACL, newAceIndex , &pTempAce)) 
		{
           _tprintf(TEXT("GetAce() failed. Error %d [19]\n"), GetLastError());
           __leave;
		}
		
		((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags = 
			OBJECT_INHERIT_ACE | INHERIT_ONLY_ACE;

        // You have a pointer to the ACE. Place it into a structure, so you can get at its size.           
		if (!AddAccessAllowedAce(pNewACL, ACL_REVISION , dwAccessMask,pUserSID)) 
		{
			_tprintf(TEXT("AddAccessAllowedAce() failed. Error %d [20]\n"),
			GetLastError());
			__leave;
		}

        //add another ACE for files

        if (!GetAce(pNewACL, newAceIndex+1, &pTempAce)) 
		{
           _tprintf(TEXT("GetAce() failed. Error %d [21]\n"),GetLastError());
           __leave;
		}
		((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags = 
			CONTAINER_INHERIT_ACE;
	 }

      // 
      // STEP 18: Set the new DACL to the new SD.
      // 
      if (!SetSecurityDescriptorDacl(&newSD, TRUE, pNewACL, FALSE)) 
	  {
         _tprintf(TEXT("SetSecurityDescriptorDacl() failed. Error %d [22]\n"),
               GetLastError());
         __leave;
      }

      // 
      // STEP 19: Copy the old security descriptor control flags 
      // regarding DACL automatic inheritance for Windows 2000 or 
      // later where SetSecurityDescriptorControl() API is available
      // in advapi32.dll.
      // 
      _SetSecurityDescriptorControl = (SetSecurityDescriptorControlFnPtr)
            GetProcAddress(GetModuleHandle(TEXT("advapi32.dll")),
            "SetSecurityDescriptorControl");
      if (_SetSecurityDescriptorControl) {

         SECURITY_DESCRIPTOR_CONTROL controlBitsOfInterest = 0;
         SECURITY_DESCRIPTOR_CONTROL controlBitsToSet = 0;
         SECURITY_DESCRIPTOR_CONTROL oldControlBits = 0;
         DWORD dwRevision = 0;

         if (!GetSecurityDescriptorControl(pFileSD, &oldControlBits,
            &dwRevision)) {
            _tprintf(TEXT("GetSecurityDescriptorControl() failed. Error %d [23]\n"), GetLastError());
            __leave;
         }

         if (oldControlBits & SE_DACL_AUTO_INHERITED) {
            controlBitsOfInterest =
               SE_DACL_AUTO_INHERIT_REQ |
               SE_DACL_AUTO_INHERITED;
            controlBitsToSet = controlBitsOfInterest;
         }
         else if (oldControlBits & SE_DACL_PROTECTED) {
            controlBitsOfInterest = SE_DACL_PROTECTED;
            controlBitsToSet = controlBitsOfInterest;
         }
         
         if (controlBitsOfInterest) {
            if (!_SetSecurityDescriptorControl(&newSD,
               controlBitsOfInterest,
               controlBitsToSet)) {
               _tprintf(TEXT("SetSecurityDescriptorControl() failed. Error %d [24]\n"), GetLastError());
               __leave;
            }
         }
      }

      // 
      // STEP 20: Set the new SD to the File.
      // 
      if (!SetFileSecurity(lpszFileName, secInfo,
            &newSD)) {
         _tprintf(TEXT("SetFileSecurity() failed. Error %d [25]\n"),GetLastError());
         __leave;
      }

  /*
  // 
	  // ADDITIONAL STEP 17b: Detection in the case of Directory use
	  //
      
	  //You have now rebuilt a new ACL and want to add it to the newly created DACL.
      if (!AddAccessAllowedAce(pNewACL, ACL_REVISION , dwAccessMask,pUserSID)) 
	  {
         _tprintf(TEXT("AddAccessAllowedAce() failed. Error %d [18]\n"),
         GetLastError());
         __leave;
      }

      //If it's directory, we need to add inheritance staff.
      if ( (GetFileAttributes(lpszFileName) & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)
	  {
        // Attempt to grab the next ACE which is what we just added.

        if (!GetAce(pNewACL, newAceIndex , &pTempAce)) 
		{
           _tprintf(TEXT("GetAce() failed. Error %d [19]\n"), 
                 GetLastError());
           __leave;
		}
		//((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags = 
		//	OBJECT_INHERIT_ACE | INHERIT_ONLY_ACE;

        // You have a pointer to the ACE. Place it into a structure, so you can get at its size.           
		if (!AddAccessAllowedAce(pNewACL, ACL_REVISION , dwAccessMask,pUserSID)) 
		{
			_tprintf(TEXT("AddAccessAllowedAce() failed. Error %d [20]\n"),
			GetLastError());
			__leave;
		}

        //add another ACE for files

        if (!GetAce(pNewACL, newAceIndex+1, &pTempAce)) 
		{
           _tprintf(TEXT("GetAce() failed. Error %d [21]\n"),GetLastError());
           __leave;
		}
		//((ACCESS_ALLOWED_ACE *)pTempAce)->Header.AceFlags = 
		//	CONTAINER_INHERIT_ACE;
	 }

*/
      fResult = TRUE;
   } __finally {

      // 
      // STEP 21: Free allocated memory
      // 
      if (pUserSID) myheapfree(pUserSID);
      if (szDomain) myheapfree(szDomain);
      if (pFileSD)  myheapfree(pFileSD);
      if (pNewACL)  myheapfree(pNewACL);
   }
   return fResult;
}




// Secure Server Sample

//   David Mowers (davemo)   15-May-97 */ 

#include <windows.h>
#include <stdio.h>

// This code must be linked with the Advapi32.lib file.

// Make up some private access rights.
#define ACCESS_READ  1
#define ACCESS_WRITE 2

void main(int argc, char *argv[]) {

   PSECURITY_DESCRIPTOR psdSD;

   // User/SID variables.
   HANDLE      hToken   = NULL;
   PTOKEN_USER ptuUser  = NULL;
   DWORD       cbBuffer = 0;
   PSID        pUserSid = NULL;

   // ACE variables.
   DWORD dwAccessMask = ACCESS_READ | ACCESS_WRITE;
   PACL  pACL         = NULL;
   DWORD dwACLSize;

   // AccessCheck() variables
   DWORD           dwAccessDesired;
   PRIVILEGE_SET   PrivilegeSet;
   DWORD           dwPrivSetSize;
   DWORD           dwAccessGranted;
   BOOL            fAccessGranted = FALSE;
   GENERIC_MAPPING GenericMapping;

   __try {

      // Get a SID for later use. A real server would use a function
      // such as LookupAccountName() to get SIDs that you will use to
      // build your access control list.
      if (!OpenProcessToken(GetCurrentProcess(), TOKEN_READ, &hToken)) {

         printf("Error %d:OpenProcessToken", GetLastError());
         __leave;
      }

      // Determine required size of buffer for token information.
      if (GetTokenInformation(hToken, TokenUser, NULL, 0, &cbBuffer)) {

         // Call should have failed due to zero-length buffer.
         __leave;
   
      } else {

         // Call should have failed due to zero-length buffer.
         if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
            __leave;
      }

      // Allocate buffer for token information.
      ptuUser = (PTOKEN_USER) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, cbBuffer);
      if (!ptuUser)
         __leave;

      if (!GetTokenInformation(hToken, TokenUser, ptuUser, cbBuffer,
            &cbBuffer))
         __leave;

      pUserSid = ptuUser->User.Sid;

      // Build a Security Descriptor.
      psdSD = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, 
            SECURITY_DESCRIPTOR_MIN_LENGTH);

      if(!InitializeSecurityDescriptor(psdSD, 
            SECURITY_DESCRIPTOR_REVISION)) {

         printf("Error %d:InitializeSecurityDescriptor\n", 
               GetLastError());
         __leave;
      }

      // Compute size needed for the ACL.
      dwACLSize = sizeof(ACCESS_ALLOWED_ACE) + 8 +
            GetLengthSid(pUserSid) - sizeof(DWORD);

      // Allocate memory for ACL.
      pACL = (PACL) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
            dwACLSize);

      // Initialize the new ACL.
      if(!InitializeAcl(pACL, dwACLSize, ACL_REVISION2)) {
         printf("Error %d:InitializeAcl\n", GetLastError());
         __leave;
      }

      // Add the access-allowed ACE to the DACL.
      if(!AddAccessAllowedAce(pACL, ACL_REVISION2, dwAccessMask, 
            pUserSid)) {

         printf("Error %d:AddAccessAllowedAce",GetLastError());
         __leave;
      }

      // Set our DACL to the SD.
      if (!SetSecurityDescriptorDacl(psdSD, TRUE, pACL, FALSE)) {
         printf("Error %d:SetSecurityDescriptorDacl", GetLastError());
         __leave;
      }

      // AccessCheck() is picky about what is in the SD. Set the group 
      // and owner using our convenient SID.
      SetSecurityDescriptorGroup(psdSD, pUserSid, FALSE);
      SetSecurityDescriptorOwner(psdSD, pUserSid, FALSE);

      // AccessCheck() requires an impersonation token.
      // For demonstration purposes, we are going to impersonate
      // ourselves. A real server would impersonate the client by using
      // ImpersonateNamedPipeClient(), RPCImpersonateClient(),
      // ImpersonateLoggedOnUser() with a token obtained through
      // LogonUser() or the SSPI API ImpersonateSecurityContext().
      ImpersonateSelf(SecurityImpersonation);

      OpenThreadToken(GetCurrentThread(), TOKEN_ALL_ACCESS, TRUE, 
            &hToken);

      // Using AccessCheck(), there are two different things we could do:
      // 
      // 1. See if we have Read/Write access to the object.
      // 
      dwAccessDesired = ACCESS_READ;

      // Initialize generic mapping structure to map all.
      memset(&GenericMapping, 0xff, sizeof(GENERIC_MAPPING));
      GenericMapping.GenericRead = ACCESS_READ;
      GenericMapping.GenericWrite = ACCESS_WRITE;
      GenericMapping.GenericExecute = 0;
      GenericMapping.GenericAll = ACCESS_READ | ACCESS_WRITE;

      // This only does something if we want to use generic access
      // rights, like GENERIC_ALL, in our call to AccessCheck().
      MapGenericMask(&dwAccessDesired, &GenericMapping);

      dwPrivSetSize = sizeof(PRIVILEGE_SET);

      // Make the AccessCheck() call.
      if(!AccessCheck(psdSD, hToken, dwAccessDesired, &GenericMapping,
            &PrivilegeSet, &dwPrivSetSize, &dwAccessGranted, 
            &fAccessGranted)) {

         printf("Error in AccessCheck : %lu\n", GetLastError());
         __leave;
   
      } else {

         if (fAccessGranted)
            printf("Access was granted using mask %lx.\n",
                  dwAccessGranted);
         else
            printf("Access was NOT granted!\n");
      }

      // 
      // 2. Determine the maximum access I am allowed.
      // 
      dwAccessDesired = MAXIMUM_ALLOWED;

      if(!AccessCheck(psdSD, hToken, dwAccessDesired, &GenericMapping,
            &PrivilegeSet, &dwPrivSetSize, &dwAccessGranted, 
            &fAccessGranted)) {

         printf("Error in AccessCheck : %lu\n", GetLastError());
         __leave;
   
      } else {

         if (fAccessGranted)
            printf("Maximum Access Allowed = %lx\n", dwAccessGranted);
      }

      RevertToSelf();
    
   } __finally {

      // Close handles and free heap allocations.

      if (hToken)
         CloseHandle(hToken);

      if (ptuUser)
         HeapFree(GetProcessHeap(), 0, ptuUser);

      if (pACL)
         HeapFree(GetProcessHeap(), 0, pACL);

      if (psdSD)
         HeapFree(GetProcessHeap(), 0, psdSD);
   }

} 



BOOL SetAccess (char *sUserName, char *sFileName , DWORD lMask)
   DWORD LRESULT; // Result of various API calls.
   DWORD i; // Used in looping.
   char bUserSid[255]; // This will contain your SID.
   char bTempSid[255]; // This will contain the Sid of each ACE in the ACL .
   char sSystemName[1024]; // Name of this computer system.
   DWORD lSystemNameLength; // Length of string that contains the name of this system
   DWORD lLengthUserName; // Max length of user name.
   char sUserName[255]; // String to hold the current user name.
   DWORD lUserSID; // Used to hold the SID of the current user.
   DWORD lTempSid; // Used to hold the SID of each ACE in the ACL
   DWORD lUserSIDSize; // Size of the SID.
   char sDomainName[255]; // Domain the user belongs to.
   DWORD lDomainNameLength; //Length of domain name needed.
   DWORD lSIDType; // The type of SID info we are getting back.
   DWORD sFileSD = SECURITY_DESCRIPTOR   // SD of the file we want.

   char bSDBuf[1024];// Buffer that holds the security descriptor for this file.

   DWORD lFileSDSize; // Size of the File SD.
   DWORD lSizeNeeded; // Size needed for SD for file.


   DWORD sNewSD = SECURITY_DESCRIPTOR; // New security descriptor.

   ACL sACL; // Used in grabbing the DACL from the File SD.

   DWORD lDaclPresent; // Used in grabbing the DACL from the File SD.

   DWORD lDaclDefaulted; //Used in grabbing the DACL from the File SD.

   ACL_SIZE_INFORMATION sACLInfo ;//Used in grabbing the ACL from the File SD.

   DWORD lACLSize ; // Size of the ACL structure used to get the ACL from the File SD.

   DWORD pAcl ; // Current ACL for this file.
   DWORD lNewACLSize ; // Size of new ACL to create.
   char bNewACL[255]; // Buffer to hold new ACL.
   Dim bNewACL() As Byte          ' Buffer to hold new ACL.

   Dim sCurrentACE As ACCESS_ALLOWED_ACE    ' Current ACE.
   Dim pCurrentAce As Long                  ' Our current ACE.

   Dim nRecordNumber As Long

   ' Get the SID of the user. (Refer to the MSDN for more information on SIDs
   ' and their function/purpose in the operating system.) Get the SID of this
   ' user by using the LookupAccountName API. In order to use the SID
   ' of the current user account, call the LookupAccountName API
   ' twice. The first time is to get the required sizes of the SID
   ' and the DomainName string. The second call is to actually get
   ' the desired information.

   lResult = LookupAccountName(vbNullString, sUserName, _
      bUserSid(0), 255, sDomainName, lDomainNameLength, _
      lSIDType)

   ' Now set the sDomainName string buffer to its proper size before
   ' calling the API again.
   sDomainName = Space(lDomainNameLength)

   ' Call the LookupAccountName again to get the actual SID for user.
   lResult = LookupAccountName(vbNullString, sUserName, _
      bUserSid(0), 255, sDomainName, lDomainNameLength, _
      lSIDType)

   ' Return value of zero means the call to LookupAccountName failed;
   ' test for this before you continue.
     If (lResult = 0) Then
        MsgBox "Error: Unable to Lookup the Current User Account: " _
           & sUserName
        Exit Sub
     End If

   ' You now have the SID for the user who is logged on.
   ' The SID is of interest since it will get the security descriptor
   ' for the file that the user is interested in.
   ' The GetFileSecurity API will retrieve the Security Descriptor
   ' for the file. However, you must call this API twice: once to get
   ' the proper size for the Security Descriptor and once to get the
   ' actual Security Descriptor information.

   lResult = GetFileSecurityN(sFileName, DACL_SECURITY_INFORMATION, _
      0, 0, lSizeNeeded)

   ' Redimension the Security Descriptor buffer to the proper size.
   ReDim bSDBuf(lSizeNeeded)

   ' Now get the actual Security Descriptor for the file.
   lResult = GetFileSecurity(sFileName, DACL_SECURITY_INFORMATION, _
      bSDBuf(0), lSizeNeeded, lSizeNeeded)

   ' A return code of zero means the call failed; test for this
   ' before continuing.
   If (lResult = 0) Then
      MsgBox "Error: Unable to Get the File Security Descriptor"
      Exit Sub
   End If

   ' Call InitializeSecurityDescriptor to build a new SD for the
   ' file.
   lResult = InitializeSecurityDescriptor(sNewSD, _
      SECURITY_DESCRIPTOR_REVISION)

   ' A return code of zero means the call failed; test for this
   ' before continuing.
   If (lResult = 0) Then
      MsgBox "Error: Unable to Initialize New Security Descriptor"
      Exit Sub
   End If

   ' You now have the file's SD and a new Security Descriptor
   ' that will replace the current one. Next, pull the DACL from
   ' the SD. To do so, call the GetSecurityDescriptorDacl API
   ' function.

   lResult = GetSecurityDescriptorDacl(bSDBuf(0), lDaclPresent, _
      pAcl, lDaclDefaulted)

   ' A return code of zero means the call failed; test for this
   ' before continuing.
   If (lResult = 0) Then
      MsgBox "Error: Unable to Get DACL from File Security " _
         & "Descriptor"
      Exit Sub
   End If

   ' You have the file's SD, and want to now pull the ACL from the
   ' SD. To do so, call the GetACLInformation API function.
   ' See if ACL exists for this file before getting the ACL
   ' information.
   If (lDaclPresent = False) Then
      MsgBox "Error: No ACL Information Available for this File"
      Exit Sub
   End If

   ' Attempt to get the ACL from the file's Security Descriptor.
   lResult = GetAclInformation(pAcl, sACLInfo, Len(sACLInfo), 2&)

   ' A return code of zero means the call failed; test for this
   ' before continuing.
   If (lResult = 0) Then
      MsgBox "Error: Unable to Get ACL from File Security Descriptor"
      Exit Sub
   End If

   ' Now that you have the ACL information, compute the new ACL size
   ' requirements.
   lNewACLSize = sACLInfo.AclBytesInUse + (Len(sCurrentACE) + _
      GetLengthSid(bUserSid(0))) * 2 - 4

   ' Resize our new ACL buffer to its proper size.
   ReDim bNewACL(lNewACLSize)

   ' Use the InitializeAcl API function call to initialize the new
   ' ACL.
   lResult = InitializeAcl(bNewACL(0), lNewACLSize, ACL_REVISION)

   ' A return code of zero means the call failed; test for this
   ' before continuing.
   If (lResult = 0) Then
      MsgBox "Error: Unable to Initialize New ACL"
      Exit Sub
   End If

   ' If a DACL is present, copy it to a new DACL.
   If (lDaclPresent) Then

      ' Copy the ACEs from the file to the new ACL.
      If (sACLInfo.AceCount > 0) Then

         ' Grab each ACE and stuff them into the new ACL.
         nRecordNumber = 0
         For I = 0 To (sACLInfo.AceCount - 1)

            ' Attempt to grab the next ACE.
            lResult = GetAce(pAcl, I, pCurrentAce)

            ' Make sure you have the current ACE under question.
            If (lResult = 0) Then
               MsgBox "Error: Unable to Obtain ACE (" & I & ")"
               Exit Sub
            End If

            ' You have a pointer to the ACE. Place it
            ' into a structure, so you can get at its size.
            CopyMemory sCurrentACE, pCurrentAce, LenB(sCurrentACE)

            'Skip adding the ACE to the ACL if this is same usersid
            lTempSid = pCurrentAce + 8
            If EqualSid(bUserSid(0), lTempSid) = 0 Then

                ' Now that you have the ACE, add it to the new ACL.
                lResult = AddAce(VarPtr(bNewACL(0)), ACL_REVISION, _
                  MAXDWORD, pCurrentAce, _
                  sCurrentACE.Header.AceSize)

                 ' Make sure you have the current ACE under question.
                 If (lResult = 0) Then
                   MsgBox "Error: Unable to Add ACE to New ACL"
                    Exit Sub
                 End If
                 nRecordNumber = nRecordNumber + 1
            End If

         Next I

         ' You have now rebuilt a new ACL and want to add it to
         ' the newly created DACL.
         lResult = AddAccessAllowedAce(bNewACL(0), ACL_REVISION, _
            lMask, bUserSid(0))

         ' Make sure added the ACL to the DACL.
         If (lResult = 0) Then
            MsgBox "Error: Unable to Add ACL to DACL"
            Exit Sub
         End If

         'If it's directory, we need to add inheritance staff.
         If GetAttr(sFileName) And vbDirectory Then

            ' Attempt to grab the next ACE which is what we just added.
            lResult = GetAce(VarPtr(bNewACL(0)), nRecordNumber, pCurrentAce)

            ' Make sure you have the current ACE under question.
            If (lResult = 0) Then
               MsgBox "Error: Unable to Obtain ACE (" & I & ")"
               Exit Sub
            End If
            ' You have a pointer to the ACE. Place it
            ' into a structure, so you can get at its size.
            CopyMemory sCurrentACE, pCurrentAce, LenB(sCurrentACE)
            sCurrentACE.Header.AceFlags = OBJECT_INHERIT_ACE + INHERIT_ONLY_ACE
            CopyMemory ByVal pCurrentAce, VarPtr(sCurrentACE), LenB(sCurrentACE)

            'add another ACE for files
            lResult = AddAccessAllowedAce(bNewACL(0), ACL_REVISION, _
               lMask, bUserSid(0))

            ' Make sure added the ACL to the DACL.
            If (lResult = 0) Then
               MsgBox "Error: Unable to Add ACL to DACL"
               Exit Sub
            End If

            ' Attempt to grab the next ACE.
            lResult = GetAce(VarPtr(bNewACL(0)), nRecordNumber + 1, pCurrentAce)

            ' Make sure you have the current ACE under question.
            If (lResult = 0) Then
               MsgBox "Error: Unable to Obtain ACE (" & I & ")"
               Exit Sub
            End If

            CopyMemory sCurrentACE, pCurrentAce, LenB(sCurrentACE)
            sCurrentACE.Header.AceFlags = CONTAINER_INHERIT_ACE
            CopyMemory ByVal pCurrentAce, VarPtr(sCurrentACE), LenB(sCurrentACE)
        End If


         ' Set the file's Security Descriptor to the new DACL.
         lResult = SetSecurityDescriptorDacl(sNewSD, 1, _
            bNewACL(0), 0)

         ' Make sure you set the SD to the new DACL.
         If (lResult = 0) Then
            MsgBox "Error: " & _
                "Unable to Set New DACL to Security Descriptor"
            Exit Sub
         End If

         ' The final step is to add the Security Descriptor back to
         ' the file!
         lResult = SetFileSecurity(sFileName, _
            DACL_SECURITY_INFORMATION, sNewSD)

         ' Make sure you added the Security Descriptor to the file!
         If (lResult = 0) Then
            MsgBox "Error: Unable to Set New Security Descriptor " _
               & " to File : " & sFileName
            MsgBox Err.LastDllError
         Else
            MsgBox "Updated Security Descriptor on File: " _
               & sFileName
         End If

      End If

   End If

End Sub
